/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/*
 *    Brutus source file for ImageMagick powered graphic format
 *    conversion helper functions.
 *
 *    Copyright (C) 2007 OMC Denmark ApS.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "brutus_jpeg.h"

#include <stdlib.h>
#include <string.h>
#include <wand/magick-wand.h>

void*
brutus_any_graphics_to_jpeg(const size_t inbuf_len,
			    const void *inbuf,
			    size_t *jpeg_len)
{
	MagickWand *wand = 0;
	MagickBooleanType magick_retv;
	size_t wand_blob_len = 0;
	unsigned char *wand_blob = NULL;
	void *jpeg_buf = NULL;

	MagickWandGenesis();
	wand = NewMagickWand();

	*jpeg_len = 0;

	magick_retv = MagickReadImageBlob(wand,
					  inbuf,
					  inbuf_len);
	if (MagickFalse == magick_retv)
		goto exit_convert;

	magick_retv = MagickSetImageFormat(wand, "JPEG");
	if (MagickFalse == magick_retv)
		goto exit_convert;

	wand_blob = MagickGetImageBlob(wand, &wand_blob_len);
	if (!wand_blob)
		goto exit_convert;

	jpeg_buf = malloc(wand_blob_len);
	if (!jpeg_buf)
		goto exit_convert;

	*jpeg_len = wand_blob_len;
	memcpy(jpeg_buf, wand_blob, wand_blob_len);

exit_convert:
	if (wand_blob)
		MagickRelinquishMemory(wand_blob);

	wand = DestroyMagickWand(wand);
	MagickWandTerminus();

	return jpeg_buf;
}

